#include <spawn.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/wait.h>
#include <unistd.h>

#include "error.h"
#include "run.h"
#include "util.h"

extern char **environ;

char *run_find_cmd(char *cmd) {
  char *path = getenv("PATH"), *save = NULL, *el = NULL;

  if ((el = strtok_r(path, ":", &save)) == NULL)
    return NULL;

  do {
    char fp[strlen(el) + strlen(cmd) + 2];
    join(fp, el, cmd);

    if (exists(fp))
      return strdup(fp);
  } while ((el = strtok_r(NULL, ":", &save)) != NULL);

  return NULL;
}

bool run_cmd(char **all) {
  int   status = 0;
  pid_t pid    = 0;

  if (posix_spawn(&pid, all[0], NULL, NULL, all, environ) != 0)
    return false;

  waitpid(pid, &status, 0);
  return status == 0;
}

bool run_cmd_root(char *cmd, char **args) {
  char *rootcmd = NULL;

  if (NULL == run_find_cmd(cmd)) {
    error_set("Command not found");
    errno = RunCmdNotFound;
    return false;
  }

  if (getuid() == 0)
    goto RUN;
  else if ((rootcmd = run_find_cmd("doas")) != NULL)
    goto RUN;
  else if ((rootcmd = run_find_cmd("sudo")) != NULL)
    goto RUN;

  error_set("There is no doas or sudo installed");
  errno = RunNoRoot;
  return false;

RUN:
  errno = 0;

  int arg_count = 0;
  while (args[arg_count] != NULL)
    arg_count++;
  arg_count += 2;

  if (rootcmd != NULL)
    arg_count++;

  char **all  = malloc(arg_count * sizeof(char *));
  int    indx = 0;

  if (rootcmd != NULL) {
    all[indx]   = rootcmd;
    all[++indx] = cmd;
  } else
    all[indx] = rootcmd;

  for (int i = 0; i < arg_count; i++)
    all[++indx] = args[i];
  all[++indx] = NULL;

  bool ret = true;

  if (!run_cmd(all)) {
    error_set("Command failed");
    errno = RunCmdFail;
    ret   = false;
  }

  free(all);
  return ret;
}
